package com.brandwatch.microservice.client;

import static com.google.common.collect.Lists.newArrayList;
import static java.util.concurrent.TimeUnit.SECONDS;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.guava.GuavaCache;
import org.springframework.cache.support.SimpleCacheManager;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.EnableAspectJAutoProxy;
import org.springframework.context.annotation.Import;

import com.google.common.cache.CacheBuilder;
import com.netflix.hystrix.contrib.javanica.aop.aspectj.HystrixCommandAspect;

import com.brandwatch.microservice.client.helpers.ExampleRemoteServiceRegistry;

@Configuration
@ComponentScan
@EnableAutoConfiguration
@Import(ExampleRemoteServiceRegistry.class)
@EnableAspectJAutoProxy
@EnableCaching
public class DemoMicroserviceClientApplication {
    
    public static void main(String[] args) {
        SpringApplication.run(DemoMicroserviceClientApplication.class, args);
    }
    
//    @Bean
//    public BeanFactory echoRepository() {
//        HttpInvokerProxyFactoryBean proxyFactory = new HttpInvokerProxyFactoryBean();
//        proxyFactory.setServiceInterface(EchoRepository.class);
//        proxyFactory.setServiceUrl("http://localhost:9000/echoService");
//        return (BeanFactory) proxyFactory;
//    } 

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    @Bean
    public HystrixCommandAspect hystrixAspect() {
        return new HystrixCommandAspect();
    }
    
    @Bean
    public CacheManager cacheManager() {
        SimpleCacheManager simpleCacheManager = new SimpleCacheManager();

        GuavaCache projectClientIdCache = new GuavaCache("echoCache",
                                                         CacheBuilder
                                                            .newBuilder()
                                                            .expireAfterWrite(10, SECONDS)
                                                            .build());
        
        simpleCacheManager.setCaches(newArrayList(projectClientIdCache));
        
        return simpleCacheManager;
    }

}
