package com.brandwatch.microservice.client.helpers;

import static org.springframework.beans.factory.support.AbstractBeanDefinition.AUTOWIRE_BY_NAME;

import java.util.LinkedList;
import java.util.List;

import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.GenericBeanDefinition;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.ImportBeanDefinitionRegistrar;
import org.springframework.core.env.Environment;
import org.springframework.core.type.AnnotationMetadata;
import org.springframework.remoting.httpinvoker.HttpInvokerProxyFactoryBean;
import org.springframework.stereotype.Component;

@Component
public abstract class RemoteServiceRegistry implements ImportBeanDefinitionRegistrar, EnvironmentAware {

    private Environment environment;

    private final List<RemoteServiceDefinition> remoteServices = new LinkedList<RemoteServiceDefinition>();
    boolean serviceListPopulated = false;

    public abstract void registerServices();

    @Override
    public void registerBeanDefinitions(AnnotationMetadata am, BeanDefinitionRegistry bdr) {
        if (!serviceListPopulated) {
            registerServices();
            serviceListPopulated = true;
        }

        for (RemoteServiceDefinition remoteService : remoteServices) {
            GenericBeanDefinition beanDefinition = new GenericBeanDefinition();
            beanDefinition.setBeanClass(HttpInvokerProxyFactoryBean.class);
            beanDefinition.getPropertyValues().addPropertyValue("serviceUrl", remoteService.getServiceUrl());
            beanDefinition.getPropertyValues().addPropertyValue("serviceInterface", remoteService.getServiceInterface());
            beanDefinition.setAutowireMode(AUTOWIRE_BY_NAME);

            bdr.registerBeanDefinition(remoteService.beanName, beanDefinition);
        }
    }

    protected void registerService(String beanName, String serviceUrl, Class<?> serviceInterface) {
        RemoteServiceDefinition remoteServiceDefinition = new RemoteServiceDefinition(beanName, serviceUrl, serviceInterface);
        remoteServices.add(remoteServiceDefinition);
    }

    protected void registerService(String serviceUrl, Class<?> serviceInterface) {
        RemoteServiceDefinition remoteServiceDefinition = new RemoteServiceDefinition(serviceUrl, serviceInterface);
        remoteServices.add(remoteServiceDefinition);
    }

    protected void registerServiceFromProperty(String beanName, String serviceUrlPropertyName, Class<?> serviceInterface) {
        String serviceUrl = resolveProperty(serviceUrlPropertyName);
        RemoteServiceDefinition remoteServiceDefinition = new RemoteServiceDefinition(beanName, serviceUrl, serviceInterface);
        remoteServices.add(remoteServiceDefinition);
    }

    private String resolveProperty(String propertyName) {
        String property = environment.getProperty(propertyName);
        if (property == null) {
            throw new IllegalArgumentException("Unable to resolve property " + propertyName);
        } else {
            return property;
        }
    }

    protected void registerServiceFromProperty(String serviceUrlPropertyName, Class<?> serviceInterface) {
        String serviceUrl = resolveProperty(serviceUrlPropertyName);
        RemoteServiceDefinition remoteServiceDefinition = new RemoteServiceDefinition(serviceUrl, serviceInterface);
        remoteServices.add(remoteServiceDefinition);
    }

    @Override
    public void setEnvironment(Environment environment) {
        this.environment = environment;
    }

}
