package com.brandwatch.microservice.client.services;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import com.netflix.hystrix.contrib.javanica.annotation.HystrixCommand;
import com.netflix.hystrix.contrib.javanica.annotation.HystrixProperty;

import com.brandwatch.microservice.interfaces.EchoRepository;

@Component
public class SafeEchoService {

    @Autowired
    private EchoRepository echoRepository;

    @HystrixCommand(commandProperties = {
        @HystrixProperty(name = "execution.isolation.thread.timeoutInMilliseconds", value = "500")
    },
    fallbackMethod = "echoFallback")
    @Cacheable("echoCache")
    public String echo(String string) {
        return echoRepository.echo(string);
    }
    
    public String echoFallback(String string) {
        return "Sorry can't echo " + string + " at the moment :-(";
    }

}
