package com.brandwatch.microservice.server.helpers;

import static org.springframework.beans.factory.support.AbstractBeanDefinition.AUTOWIRE_NO;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.GenericBeanDefinition;
import org.springframework.context.annotation.ClassPathScanningCandidateComponentProvider;
import org.springframework.context.annotation.ImportBeanDefinitionRegistrar;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.core.type.AnnotationMetadata;
import org.springframework.core.type.filter.AnnotationTypeFilter;
import org.springframework.remoting.httpinvoker.HttpInvokerServiceExporter;

import com.brandwatch.microservice.server.annotations.RemoteService;

public class ServiceExporter implements ImportBeanDefinitionRegistrar, BeanFactoryAware {
    
    private final Logger log = LoggerFactory.getLogger(ServiceExporter.class);
    
    private BeanFactory beanFactory;
    
    @Override
    public void registerBeanDefinitions(AnnotationMetadata am, BeanDefinitionRegistry bdr) {
        
        ClassPathScanningCandidateComponentProvider scanner
                                                    = new ClassPathScanningCandidateComponentProvider(false);

        scanner.addIncludeFilter(new AnnotationTypeFilter(RemoteService.class));
        
        for (BeanDefinition bd : scanner.findCandidateComponents("com.brandwatch")) {
            try {
                String beanClassName = bd.getBeanClassName();
                Class<?> beanClass = Class.forName(beanClassName);
                RemoteService annotation = AnnotationUtils.findAnnotation(beanClass, RemoteService.class);
                
                Object instance = beanFactory.getBean(beanClass);
                Class<?> supportedService = annotation.value();
                
                if (!supportedService.isInstance(instance)) {
                    log.error("{} does not support the interface {} specified in it's remote service annotation. Skipping.", beanClass, supportedService.getName());
                    continue;
                }
                
                GenericBeanDefinition beanDefinition = new GenericBeanDefinition();
                beanDefinition.setBeanClass(HttpInvokerServiceExporter.class);
                beanDefinition.getPropertyValues().addPropertyValue("service", instance);
                beanDefinition.getPropertyValues().addPropertyValue("serviceInterface", supportedService);
                beanDefinition.setAutowireMode(AUTOWIRE_NO);
                
                bdr.registerBeanDefinition("/remoting/" + supportedService.getName(), beanDefinition);
            } catch (ClassNotFoundException ex) {
                log.error("Unable to find class definition for class found during package scan... This should never happen so check your class loader!", ex);
            }
        }
    }

    @Override
    public void setBeanFactory(BeanFactory beanFactory) throws BeansException {
        this.beanFactory = beanFactory;
    }

}
